@@ -63,13 +63,47 @@ namespace Breeze
         if (auto d = qobject_cast<Decoration*>(decoration))
         {
             Button *b = new Button(type, d, parent);
-            if (type == KDecoration2::DecorationButtonType::Menu)
+            switch( type )
             {
+
+                case KDecoration2::DecorationButtonType::Close:
+                b->setVisible( d->client().data()->isCloseable() );
+                QObject::connect(d->client().data(), &KDecoration2::DecoratedClient::closeableChanged, b, &Breeze::Button::setVisible );
+                break;
+
+                case KDecoration2::DecorationButtonType::Maximize:
+                b->setVisible( d->client().data()->isMaximizeable() );
+                QObject::connect(d->client().data(), &KDecoration2::DecoratedClient::maximizeableChanged, b, &Breeze::Button::setVisible );
+                break;
+
+                case KDecoration2::DecorationButtonType::Minimize:
+                b->setVisible( d->client().data()->isMinimizeable() );
+                QObject::connect(d->client().data(), &KDecoration2::DecoratedClient::minimizeableChanged, b, &Breeze::Button::setVisible );
+                break;
+
+                case KDecoration2::DecorationButtonType::ContextHelp:
+                b->setVisible( d->client().data()->providesContextHelp() );
+                QObject::connect(d->client().data(), &KDecoration2::DecoratedClient::providesContextHelpChanged, b, &Breeze::Button::setVisible );
+                break;
+
+                case KDecoration2::DecorationButtonType::Shade:
+                b->setVisible( d->client().data()->isShadeable() );
+                QObject::connect(d->client().data(), &KDecoration2::DecoratedClient::shadeableChanged, b, &Breeze::Button::setVisible );
+                break;
+
+                case KDecoration2::DecorationButtonType::Menu:
                 QObject::connect(d->client().data(), &KDecoration2::DecoratedClient::iconChanged, b, [b]() { b->update(); });
+                break;
+
+                default: break;
+
             }
+
             return b;
         }
+
         return nullptr;
+
     }
 
     //__________________________________________________________________
@@ -141,8 +175,7 @@ namespace Breeze
             QPen pen( foregroundColor );
             pen.setCapStyle( Qt::RoundCap );
             pen.setJoinStyle( Qt::MiterJoin );
-            const qreal penWidth( 1 );
-            pen.setWidth( penWidth*2 );
+            pen.setWidthF( 1.1*qMax(1.0, 20/width ) );
 
             painter->setPen( pen );
             painter->setBrush( Qt::NoBrush );
@@ -152,8 +185,8 @@ namespace Breeze
 
                 case KDecoration2::DecorationButtonType::Close:
                 {
-                    painter->drawLine( QPointF( 5 + penWidth, 5 + penWidth ), QPointF( 13 - penWidth, 13 - penWidth ) );
-                    painter->drawLine( 13 - penWidth, 5 + penWidth, 5 + penWidth, 13 - penWidth );
+                    painter->drawLine( QPointF( 5, 5 ), QPointF( 13, 13 ) );
+                    painter->drawLine( 13, 5, 5, 13 );
                     break;
                 }
 
@@ -161,35 +194,29 @@ namespace Breeze
                 {
                     if( isChecked() )
                     {
-
                         pen.setJoinStyle( Qt::RoundJoin );
                         painter->setPen( pen );
-
                         painter->drawPolygon( QPolygonF()
-                            << QPointF( 3.5 + penWidth, 9 )
-                            << QPointF( 9, 3.5 + penWidth )
-                            << QPointF( 14.5 - penWidth, 9 )
-                            << QPointF( 9, 14.5 - penWidth ) );
-
+                            << QPointF( 4.5, 9 )
+                            << QPointF( 9, 4.5 )
+                            << QPointF( 13.5, 9 )
+                            << QPointF( 9, 13.5 ) );
                     } else {
-
                         painter->drawPolyline( QPolygonF()
-                            << QPointF( 3.5 + penWidth, 11.5 - penWidth )
-                            << QPointF( 9, 5.5 + penWidth )
-                            << QPointF( 14.5 - penWidth, 11.5 - penWidth ) );
+                            << QPointF( 4, 11 )
+                            << QPointF( 9, 6 )
+                            << QPointF( 14, 11 ) );
                     }
                     break;
                 }
 
                 case KDecoration2::DecorationButtonType::Minimize:
                 {
-
                     painter->drawPolyline( QPolygonF()
-                        << QPointF( 3.5 + penWidth, 6.5 + penWidth )
-                        << QPointF( 9, 12.5 - penWidth )
-                        << QPointF( 14.5 - penWidth, 6.5 + penWidth ) );
+                        << QPointF( 4, 7 )
+                        << QPointF( 9, 12 )
+                        << QPointF( 14, 7 ) );
                     break;
-
                 }
 
                 case KDecoration2::DecorationButtonType::OnAllDesktops:
@@ -216,10 +243,15 @@ namespace Breeze
 
                     } else {
 
-                        painter->drawRoundedRect( QRectF( 6, 2, 6, 9 ), 1.5, 1.5 );
-                        painter->drawRect( QRectF( 4, 10, 10, 2 ) );
-                        painter->drawRoundRect( QRectF( 8, 12, 2, 4 ) );
+                        painter->drawPolygon( QPolygonF()
+                            << QPointF( 6.5, 8.5 )
+                            << QPointF( 12, 3 )
+                            << QPointF( 15, 6 )
+                            << QPointF( 9.5, 11.5 ) );
 
+                        painter->setPen( pen );
+                        painter->drawLine( QPointF( 5.5, 7.5 ), QPointF( 10.5, 12.5 ) );
+                        painter->drawLine( QPointF( 12, 6 ), QPointF( 4.5, 13.5 ) );
                     }
                     break;
                 }
@@ -230,19 +262,19 @@ namespace Breeze
                     if (isChecked())
                     {
 
-                        painter->drawLine( 3 + penWidth, 5.5 + penWidth, 15 - penWidth, 5.5+penWidth );
+                        painter->drawLine( 4, 5, 14, 5 );
                         painter->drawPolyline( QPolygonF()
-                            << QPointF( 3.5 + penWidth, 8.5 + penWidth )
-                            << QPointF( 9, 14.5 - penWidth )
-                            << QPointF( 14.5 - penWidth, 8.5 + penWidth ) );
+                            << QPointF( 4, 8 )
+                            << QPointF( 9, 13 )
+                            << QPointF( 14, 8 ) );
 
                     } else {
 
-                        painter->drawLine( 3 + penWidth, 5.5 + penWidth, 15 - penWidth, 5.5+penWidth );
+                        painter->drawLine( 4, 5, 14, 5 );
                         painter->drawPolyline( QPolygonF()
-                            << QPointF( 3.5 + penWidth, 14.5 - penWidth )
-                            << QPointF( 9, 8.5 + penWidth )
-                            << QPointF( 14.5 - penWidth, 14.5 - penWidth ) );
+                            << QPointF( 4, 13 )
+                            << QPointF( 9, 8 )
+                            << QPointF( 14, 13 ) );
                     }
 
                     break;
@@ -253,14 +285,14 @@ namespace Breeze
                 {
 
                     painter->drawPolyline( QPolygonF()
-                        << QPointF( 3.5 + penWidth, 4.5 + penWidth )
-                        << QPointF( 9, 10.5 - penWidth )
-                        << QPointF( 14.5 - penWidth, 4.5 + penWidth ) );
+                        << QPointF( 4, 5 )
+                        << QPointF( 9, 10 )
+                        << QPointF( 14, 5 ) );
 
                     painter->drawPolyline( QPolygonF()
-                        << QPointF( 3.5 + penWidth, 8.5 + penWidth )
-                        << QPointF( 9, 14.5 - penWidth )
-                        << QPointF( 14.5 - penWidth, 8.5 + penWidth ) );
+                        << QPointF( 4, 9 )
+                        << QPointF( 9, 14 )
+                        << QPointF( 14, 9 ) );
                     break;
 
                 }
@@ -268,17 +300,18 @@ namespace Breeze
                 case KDecoration2::DecorationButtonType::KeepAbove:
                 {
                     painter->drawPolyline( QPolygonF()
-                        << QPointF( 3.5 + penWidth, 9.5 - penWidth )
-                        << QPointF( 9, 3.5 + penWidth )
-                        << QPointF( 14.5 - penWidth, 9.5 - penWidth ) );
+                        << QPointF( 4, 9 )
+                        << QPointF( 9, 4 )
+                        << QPointF( 14, 9 ) );
 
                     painter->drawPolyline( QPolygonF()
-                        << QPointF( 3.5 + penWidth, 13.5 - penWidth )
-                        << QPointF( 9, 7.5 + penWidth )
-                        << QPointF( 14.5 - penWidth, 13.5 - penWidth ) );
+                        << QPointF( 4, 13 )
+                        << QPointF( 9, 8 )
+                        << QPointF( 14, 13 ) );
                     break;
                 }
 
+
                 case KDecoration2::DecorationButtonType::ApplicationMenu:
                 {
                     painter->drawLine( QPointF( 3.5, 5 ), QPointF( 14.5, 5 ) );
@@ -311,10 +344,16 @@ namespace Breeze
     //__________________________________________________________________
     QColor Button::foregroundColor( void ) const
     {
-
         auto d = qobject_cast<Decoration*>( decoration() );
-        if( !d ) return QColor();
-        if( type() == KDecoration2::DecorationButtonType::Close ) {
+        if( !d ) {
+
+            return QColor();
+
+        } else if( isPressed() ) {
+
+            return d->titleBarColor();
+
+        } else if( type() == KDecoration2::DecorationButtonType::Close && d->internalSettings()->outlineCloseButton() ) {
 
             return d->titleBarColor();
 
@@ -342,9 +381,11 @@ namespace Breeze
     QColor Button::backgroundColor( void ) const
     {
         auto d = qobject_cast<Decoration*>( decoration() );
-        if( !d ) return QColor();
-        if( isPressed() )
-        {
+        if( !d ) {
+
+            return QColor();
+
+        } else if( isPressed() ) {
 
             if( type() == KDecoration2::DecorationButtonType::Close ) return d->colorSettings().closeButtonColor();
             else return KColorUtils::mix( d->titleBarColor(), d->fontColor(), 0.3 );
@@ -357,8 +398,18 @@ namespace Breeze
 
             if( type() == KDecoration2::DecorationButtonType::Close )
             {
+                if( d->internalSettings()->outlineCloseButton() )
+                {
 
-                return KColorUtils::mix( d->fontColor(), d->colorSettings().closeButtonColor().lighter(), m_opacity );
+                    return KColorUtils::mix( d->fontColor(), d->colorSettings().closeButtonColor().lighter(), m_opacity );
+
+                } else {
+
+                    QColor color( d->colorSettings().closeButtonColor().lighter() );
+                    color.setAlpha( color.alpha()*m_opacity );
+                    return color;
+
+                }
 
             } else {
 
@@ -373,7 +424,7 @@ namespace Breeze
             if( type() == KDecoration2::DecorationButtonType::Close ) return d->colorSettings().closeButtonColor().lighter();
             else return d->fontColor();
 
-        } else if( type() == KDecoration2::DecorationButtonType::Close ) {
+        } else if( type() == KDecoration2::DecorationButtonType::Close && d->internalSettings()->outlineCloseButton() ) {
 
             return d->fontColor();
 
