@@ -24,7 +24,7 @@
 #include "breeze.h"
 #include "breezesettingsprovider.h"
 #include "config-breeze.h"
-#include "config/breezeconfig.h"
+#include "config/breezeconfigwidget.h"
 
 #include "breezebutton.h"
 #include "breezesizegrip.h"
@@ -46,12 +46,14 @@
 #include <QX11Info>
 #endif
 
+#include <cmath>
+
 K_PLUGIN_FACTORY_WITH_JSON(
     BreezeDecoFactory,
     "breeze.json",
     registerPlugin<Breeze::Decoration>();
     registerPlugin<Breeze::Button>(QStringLiteral("button"));
-    registerPlugin<Breeze::ConfigurationModule>(QStringLiteral("kcmodule"));
+    registerPlugin<Breeze::ConfigWidget>(QStringLiteral("kcmodule"));
 )
 
 namespace Breeze
@@ -60,15 +62,17 @@ namespace Breeze
 
     //________________________________________________________________
     static int g_sDecoCount = 0;
+    static int g_shadowSize = 0;
+    static int g_shadowStrength = 0;
     static QSharedPointer<KDecoration2::DecorationShadow> g_sShadow;
 
+    //________________________________________________________________
     Decoration::Decoration(QObject *parent, const QVariantList &args)
         : KDecoration2::Decoration(parent, args)
-        , m_colorSettings(client().data()->palette())
+        , m_colorSettings()
         , m_animation( new QPropertyAnimation( this ) )
     {
-        g_sDecoCount++;
-        m_useSeparator = (m_colorSettings.palette().color( QPalette::Window ) != m_colorSettings.activeTitleBar() );
+        // g_sDecoCount++;
     }
 
     //________________________________________________________________
@@ -98,7 +102,8 @@ namespace Breeze
     QColor Decoration::titleBarColor() const
     {
 
-        if( m_animation->state() == QPropertyAnimation::Running )
+        if( hideTitleBar() ) return m_colorSettings.titleBar( false );
+        else if( m_animation->state() == QPropertyAnimation::Running )
         {
             return KColorUtils::mix( m_colorSettings.inactiveTitleBar(), m_colorSettings.activeTitleBar(), m_opacity );
         } else return m_colorSettings.titleBar( client().data()->isActive() );
@@ -132,6 +137,9 @@ namespace Breeze
     //________________________________________________________________
     void Decoration::init()
     {
+        m_colorSettings.update(client().data()->palette(), *client().data());
+        m_useSeparator = (m_colorSettings.palette().color( QPalette::Window ) != m_colorSettings.activeTitleBar() );
+
 
         // active state change animation
         m_animation->setStartValue( 0 );
@@ -165,21 +173,21 @@ namespace Breeze
         );
 
         connect(client().data(), &KDecoration2::DecoratedClient::activeChanged, this, &Decoration::updateAnimationState);
-        connect(client().data(), &KDecoration2::DecoratedClient::paletteChanged,   this,
+        connect(client().data(), &KDecoration2::DecoratedClient::paletteChanged, this,
             [this]() {
-                m_colorSettings.update(client().data()->palette());
+                m_colorSettings.update(client().data()->palette(), *client().data());
                 m_useSeparator = (m_colorSettings.palette().color( QPalette::Window ) != m_colorSettings.activeTitleBar() );
                 update();
             }
         );
-        connect(client().data(), &KDecoration2::DecoratedClient::widthChanged,     this, &Decoration::updateTitleBar);
+        connect(client().data(), &KDecoration2::DecoratedClient::widthChanged, this, &Decoration::updateTitleBar);
         connect(client().data(), &KDecoration2::DecoratedClient::maximizedChanged, this, &Decoration::updateTitleBar);
         connect(client().data(), &KDecoration2::DecoratedClient::maximizedChanged, this, &Decoration::setOpaque);
 
-        connect(client().data(), &KDecoration2::DecoratedClient::widthChanged,     this, &Decoration::updateButtonsGeometry);
+        connect(client().data(), &KDecoration2::DecoratedClient::widthChanged, this, &Decoration::updateButtonsGeometry);
         connect(client().data(), &KDecoration2::DecoratedClient::maximizedChanged, this, &Decoration::updateButtonsGeometry);
-        connect(client().data(), &KDecoration2::DecoratedClient::shadedChanged,    this, &Decoration::recalculateBorders);
-        connect(client().data(), &KDecoration2::DecoratedClient::shadedChanged,    this, &Decoration::updateButtonsGeometry);
+        connect(client().data(), &KDecoration2::DecoratedClient::shadedChanged, this, &Decoration::recalculateBorders);
+        connect(client().data(), &KDecoration2::DecoratedClient::shadedChanged, this, &Decoration::updateButtonsGeometry);
 
         createButtons();
         createShadow();
@@ -207,6 +215,14 @@ namespace Breeze
         }
     }
 
+    //________________________________________________________________
+    void Decoration::updateSizeGripVisibility()
+    {
+        auto c = client().data();
+        if( m_sizeGrip )
+        { m_sizeGrip->setVisible( c->isResizeable() && !isMaximized() && !c->isShaded() ); }
+    }
+
     //________________________________________________________________
     int Decoration::borderSize(bool bottom) const
     {
@@ -257,15 +273,15 @@ namespace Breeze
 
         m_internalSettings = SettingsProvider::self()->internalSettings( this );
 
-        // read internal settings
-        // m_internalSettings->read();
-
         // animation
         m_animation->setDuration( m_internalSettings->animationsDuration() );
 
         // borders
         recalculateBorders();
 
+        // shadow
+        createShadow();
+
         // size grip
         if( hasNoBorders() && m_internalSettings->drawSizeGrip() ) createSizeGrip();
         else deleteSizeGrip();
@@ -278,21 +294,30 @@ namespace Breeze
         auto s = settings();
         const auto c = client().data();
         const Qt::Edges edges = c->adjacentScreenEdges();
-        int left   = isMaximizedHorizontally() || edges.testFlag(Qt::LeftEdge) ? 0 : borderSize();
-        int right  = isMaximizedHorizontally() || edges.testFlag(Qt::RightEdge) ? 0 : borderSize();
 
-        QFontMetrics fm(s->font());
-        int top = qMax(fm.boundingRect(c->caption()).height(), buttonHeight() );
+        // left, right and bottom borders
+        auto testFlag = [&]( Qt::Edge edge ) { return edges.testFlag(edge) && !m_internalSettings->drawBorderOnMaximizedWindows(); };
+        const int left   = isMaximizedHorizontally() || testFlag(Qt::LeftEdge) ? 0 : borderSize();
+        const int right  = isMaximizedHorizontally() || testFlag(Qt::RightEdge) ? 0 : borderSize();
+        const int bottom = isMaximizedVertically() || c->isShaded() || testFlag(Qt::BottomEdge) ? 0 : borderSize(true);
 
-        // padding below
-        // extra pixel is used for the active window outline
-        const int baseSize = settings()->smallSpacing();
-        top += baseSize*Metrics::TitleBar_BottomMargin + 1;
+        int top = 0;
+        if( hideTitleBar() ) top = bottom;
+        else {
 
-        // padding above
-        top += baseSize*TitleBar_TopMargin;
+            QFontMetrics fm(s->font());
+            top += qMax(fm.boundingRect(c->caption()).height(), buttonHeight() );
+
+            // padding below
+            // extra pixel is used for the active window outline
+            const int baseSize = settings()->smallSpacing();
+            top += baseSize*Metrics::TitleBar_BottomMargin + 1;
+
+            // padding above
+            top += baseSize*TitleBar_TopMargin;
+
+        }
 
-        int bottom = isMaximizedVertically() || c->isShaded() || edges.testFlag(Qt::BottomEdge) ? 0 : borderSize(true);
         setBorders(QMargins(left, top, right, bottom));
 
         // extended sizes
@@ -383,12 +408,13 @@ namespace Breeze
             painter->setBrush(m_colorSettings.frame(client().data()->isActive()));
 
             // clip away the top part
-            painter->setClipRect(0, borderTop(), size().width(), size().height() - borderTop(), Qt::IntersectClip);
+            if( !hideTitleBar() ) painter->setClipRect(0, borderTop(), size().width(), size().height() - borderTop(), Qt::IntersectClip);
+
             painter->drawRoundedRect(rect(), Metrics::Frame_FrameRadius, Metrics::Frame_FrameRadius);
             painter->restore();
         }
 
-        paintTitleBar(painter, repaintRegion);
+        if( !hideTitleBar() ) paintTitleBar(painter, repaintRegion);
     }
 
     //________________________________________________________________
@@ -467,100 +493,127 @@ namespace Breeze
 
     //________________________________________________________________
     int Decoration::captionHeight() const
-    { return borderTop() - settings()->smallSpacing()*(Metrics::TitleBar_BottomMargin + Metrics::TitleBar_TopMargin ) - 1; }
+    { return hideTitleBar() ? borderTop() : borderTop() - settings()->smallSpacing()*(Metrics::TitleBar_BottomMargin + Metrics::TitleBar_TopMargin ) - 1; }
 
     //________________________________________________________________
     QPair<QRect,Qt::Alignment> Decoration::captionRect() const
     {
+        if( hideTitleBar() ) return qMakePair( QRect(), Qt::AlignCenter );
+        else {
 
-        const int leftOffset = m_leftButtons->geometry().x() + m_leftButtons->geometry().width() + Metrics::TitleBar_SideMargin*settings()->smallSpacing();
-        const int rightOffset = size().width() - m_rightButtons->geometry().x() + Metrics::TitleBar_SideMargin*settings()->smallSpacing();
-        const int yOffset = settings()->smallSpacing()*Metrics::TitleBar_TopMargin;
-        const QRect maxRect( leftOffset, yOffset, size().width() - leftOffset - rightOffset, captionHeight() );
+            const int leftOffset = m_leftButtons->geometry().x() + m_leftButtons->geometry().width() + Metrics::TitleBar_SideMargin*settings()->smallSpacing();
+            const int rightOffset = size().width() - m_rightButtons->geometry().x() + Metrics::TitleBar_SideMargin*settings()->smallSpacing();
+            const int yOffset = settings()->smallSpacing()*Metrics::TitleBar_TopMargin;
+            const QRect maxRect( leftOffset, yOffset, size().width() - leftOffset - rightOffset, captionHeight() );
 
-        switch( m_internalSettings->titleAlignment() )
-        {
-            case Breeze::InternalSettings::AlignLeft:
-            return qMakePair( maxRect, Qt::AlignVCenter|Qt::AlignLeft );
-
-            case Breeze::InternalSettings::AlignRight:
-            return qMakePair( maxRect, Qt::AlignVCenter|Qt::AlignRight );
-
-            case Breeze::InternalSettings::AlignCenter:
-            return qMakePair( maxRect, Qt::AlignCenter );
-
-            default:
-            case Breeze::InternalSettings::AlignCenterFullWidth:
+            switch( m_internalSettings->titleAlignment() )
             {
+                case Breeze::InternalSettings::AlignLeft:
+                return qMakePair( maxRect, Qt::AlignVCenter|Qt::AlignLeft );
 
-                // full caption rect
-                const QRect fullRect = QRect( 0, yOffset, size().width(), captionHeight() );
-                QRect boundingRect( settings()->fontMetrics().boundingRect( client().data()->caption()).toRect() );
+                case Breeze::InternalSettings::AlignRight:
+                return qMakePair( maxRect, Qt::AlignVCenter|Qt::AlignRight );
 
-                // text bounding rect
-                boundingRect.setTop( yOffset );
-                boundingRect.setHeight( captionHeight() );
-                boundingRect.moveLeft( ( size().width() - boundingRect.width() )/2 );
+                case Breeze::InternalSettings::AlignCenter:
+                return qMakePair( maxRect, Qt::AlignCenter );
 
-                if( boundingRect.left() < leftOffset ) return qMakePair( maxRect, Qt::AlignVCenter|Qt::AlignLeft );
-                else if( boundingRect.right() > size().width() - rightOffset ) return qMakePair( maxRect, Qt::AlignVCenter|Qt::AlignRight );
-                else return qMakePair(fullRect, Qt::AlignCenter);
+                default:
+                case Breeze::InternalSettings::AlignCenterFullWidth:
+                {
+
+                    // full caption rect
+                    const QRect fullRect = QRect( 0, yOffset, size().width(), captionHeight() );
+                    QRect boundingRect( settings()->fontMetrics().boundingRect( client().data()->caption()).toRect() );
+
+                    // text bounding rect
+                    boundingRect.setTop( yOffset );
+                    boundingRect.setHeight( captionHeight() );
+                    boundingRect.moveLeft( ( size().width() - boundingRect.width() )/2 );
+
+                    if( boundingRect.left() < leftOffset ) return qMakePair( maxRect, Qt::AlignVCenter|Qt::AlignLeft );
+                    else if( boundingRect.right() > size().width() - rightOffset ) return qMakePair( maxRect, Qt::AlignVCenter|Qt::AlignRight );
+                    else return qMakePair(fullRect, Qt::AlignCenter);
+
+                }
 
             }
+
         }
+
     }
 
     //________________________________________________________________
     void Decoration::createShadow()
     {
-        if (g_sShadow) {
+
+        // assign global shadow if exists and parameters match
+        if( g_sShadow && g_shadowSize == m_internalSettings->shadowSize() && g_shadowStrength == m_internalSettings->shadowStrength() )
+        {
             setShadow(g_sShadow);
             return;
         }
 
+        // assign parameters
+        g_shadowSize = m_internalSettings->shadowSize();
+        g_shadowStrength = m_internalSettings->shadowStrength();
+
         // setup shadow
+        const int shadowOffset = qMax( 6*g_shadowSize/16, Metrics::Shadow_Overlap*2 );
         auto decorationShadow = QSharedPointer<KDecoration2::DecorationShadow>::create();
         decorationShadow->setPadding( QMargins(
-            Metrics::Shadow_Size-Metrics::Shadow_Offset,
-            Metrics::Shadow_Size-Metrics::Shadow_Offset,
-            Metrics::Shadow_Size,
-            Metrics::Shadow_Size ) );
+            g_shadowSize-shadowOffset,
+            g_shadowSize-shadowOffset,
+            g_shadowSize,
+            g_shadowSize ) );
 
         decorationShadow->setInnerShadowRect(QRect(
-            Metrics::Shadow_Size-Metrics::Shadow_Offset+Metrics::Shadow_Overlap,
-            Metrics::Shadow_Size-Metrics::Shadow_Offset+Metrics::Shadow_Overlap,
-            Metrics::Shadow_Offset - 2*Metrics::Shadow_Overlap,
-            Metrics::Shadow_Offset - 2*Metrics::Shadow_Overlap ) );
+            g_shadowSize-shadowOffset+Metrics::Shadow_Overlap,
+            g_shadowSize-shadowOffset+Metrics::Shadow_Overlap,
+            shadowOffset - 2*Metrics::Shadow_Overlap,
+            shadowOffset - 2*Metrics::Shadow_Overlap ) );
 
         // create image
-        QImage image(2*Metrics::Shadow_Size, 2*Metrics::Shadow_Size, QImage::Format_ARGB32_Premultiplied);
+        QImage image(2*g_shadowSize, 2*g_shadowSize, QImage::Format_ARGB32_Premultiplied);
         image.fill(Qt::transparent);
 
-        QPainter p(&image);
-        p.setCompositionMode(QPainter::CompositionMode_Source);
-
         // create gradient
-        auto gradientStopColor = [](QColor color, qreal alpha) {
-            color.setAlphaF(alpha);
+        // gaussian delta function
+        auto alpha = [](qreal x) { return std::exp( -x*x/0.15 ); };
+
+        // color calculation delta function
+        auto gradientStopColor = [](QColor color, int alpha)
+        {
+            color.setAlpha(alpha);
             return color;
         };
 
         const QColor shadowColor( m_colorSettings.palette().color( QPalette::Shadow ) );
 
-        QRadialGradient radialGradient( Metrics::Shadow_Size, Metrics::Shadow_Size, Metrics::Shadow_Size);
-        radialGradient.setColorAt(0.0,  gradientStopColor(shadowColor, 0.35));
-        radialGradient.setColorAt(0.25, gradientStopColor(shadowColor, 0.25));
-        radialGradient.setColorAt(0.5,  gradientStopColor(shadowColor, 0.13));
-        radialGradient.setColorAt(0.75, gradientStopColor(shadowColor, 0.04));
-        radialGradient.setColorAt(1.0,  gradientStopColor(shadowColor, 0.0));
+        QRadialGradient radialGradient( g_shadowSize, g_shadowSize, g_shadowSize);
+        for( int i = 0; i < 10; ++i )
+        {
+            const qreal x( qreal( i )/9 );
+            radialGradient.setColorAt(x,  gradientStopColor(shadowColor, alpha(x)*g_shadowStrength));
+        }
+
+        radialGradient.setColorAt(1, gradientStopColor( shadowColor, 0 ) );
 
         // fill
-        p.fillRect( image.rect(), radialGradient);
+        QPainter painter(&image);
+        painter.setCompositionMode(QPainter::CompositionMode_Source);
+        painter.fillRect( image.rect(), radialGradient);
+
+        // contrast pixel
+        painter.setBrush( Qt::NoBrush );
+        painter.setPen( gradientStopColor(shadowColor, g_shadowStrength) );
+        painter.setRenderHints(QPainter::Antialiasing );
+        painter.drawRoundedRect( QRect( g_shadowSize-shadowOffset, g_shadowSize-shadowOffset, shadowOffset, shadowOffset ), 3, 3 );
+        painter.end();
 
         // assign to shadow
         decorationShadow->setShadow(image);
 
-        g_sShadow = decorationShadow;
+        // g_sShadow = decorationShadow;
         setShadow(decorationShadow);
     }
 
@@ -578,10 +631,12 @@ namespace Breeze
         KDecoration2::DecoratedClient *c( client().data() );
         if( !c ) return;
 
-        if( ( c->isResizeable() && c->windowId() != 0 ) )
+        if( c->windowId() != 0 )
         {
             m_sizeGrip = new SizeGrip( this );
-            m_sizeGrip->setVisible( !( isMaximized() || c->isShaded() ) );
+            connect( client().data(), &KDecoration2::DecoratedClient::maximizedChanged, this, &Breeze::Decoration::updateSizeGripVisibility );
+            connect( client().data(), &KDecoration2::DecoratedClient::shadedChanged, this, &Breeze::Decoration::updateSizeGripVisibility );
+            connect( client().data(), &KDecoration2::DecoratedClient::resizeableChanged, this, &Breeze::Decoration::updateSizeGripVisibility );
         }
         #endif
 
