@@ -18,7 +18,6 @@
  *************************************************************************/
 
 #include "breezeshadowhelper.h"
-#include "breezeshadowhelper.moc"
 
 #include "breeze.h"
 #include "breezehelper.h"
@@ -46,8 +45,7 @@ namespace Breeze
     //_____________________________________________________
     ShadowHelper::ShadowHelper( QObject* parent, Helper& helper ):
         QObject( parent ),
-        _helper( helper ),
-        _supported( checkSupported() )
+        _helper( helper )
         #if BREEZE_HAVE_X11
         ,_gc( 0 ),
         _atom( 0 )
@@ -81,10 +79,6 @@ namespace Breeze
     //_______________________________________________________
     bool ShadowHelper::registerWidget( QWidget* widget, bool force )
     {
-
-        // do nothing if not supported
-        if( !_supported ) return false;
-
         // make sure widget is not already registered
         if( _widgets.contains( widget ) ) return false;
 
@@ -117,6 +111,19 @@ namespace Breeze
         { uninstallX11Shadows( widget ); }
     }
 
+    //_______________________________________________________
+    void ShadowHelper::loadConfig( void )
+    {
+
+        // reset
+        reset();
+
+        // update property for registered widgets
+        for( QMap<QWidget*,WId>::const_iterator iter = _widgets.constBegin(); iter != _widgets.constEnd(); ++iter )
+        { installX11Shadows( iter.key() ); }
+
+    }
+
     //_______________________________________________________
     bool ShadowHelper::eventFilter( QObject* object, QEvent* event )
     {
@@ -144,36 +151,54 @@ namespace Breeze
             const QPalette palette( QApplication::palette() );
             const QColor shadowColor( palette.color( QPalette::Shadow ) );
 
+            // metrics
+            const int shadowSize = StyleConfigData::shadowSize()*12/16;
+            const int shadowStrength = StyleConfigData::shadowStrength();
+            const int shadowOffset = qMax( shadowSize/2, Metrics::Shadow_Overlap*2 );
+
             // pixmap
-            QPixmap pixmap = _helper.highDpiPixmap( Metrics::Shadow_Size*2 );
+            QPixmap pixmap = _helper.highDpiPixmap( shadowSize*2 );
             pixmap.fill( Qt::transparent );
 
-            // gradient
-            auto gradientStopColor = [](QColor color, qreal alpha) {
-                color.setAlphaF(alpha);
+            // create gradient
+            // gaussian delta function
+            auto alpha = [](qreal x) { return std::exp( -x*x/0.15 ); };
+
+            // color calculation delta function
+            auto gradientStopColor = [](QColor color, int alpha)
+            {
+                color.setAlpha(alpha);
                 return color;
             };
 
-            QRadialGradient radialGradient( Metrics::Shadow_Size, Metrics::Shadow_Size, Metrics::Shadow_Size);
-            radialGradient.setColorAt(0.0,  gradientStopColor( shadowColor, 0.35 ) );
-            radialGradient.setColorAt(0.25, gradientStopColor( shadowColor, 0.25 ) );
-            radialGradient.setColorAt(0.5,  gradientStopColor( shadowColor, 0.13 ) );
-            radialGradient.setColorAt(0.75, gradientStopColor( shadowColor, 0.04 ) );
-            radialGradient.setColorAt(1.0,  gradientStopColor( shadowColor, 0.0 ) );
+            QRadialGradient radialGradient( shadowSize, shadowSize, shadowSize);
+            for( int i = 0; i < 10; ++i )
+            {
+                const qreal x( qreal( i )/9 );
+                radialGradient.setColorAt(x,  gradientStopColor(shadowColor, alpha(x)*shadowStrength));
+            }
 
-            // render
-            QPainter painter( &pixmap );
-            painter.setRenderHint( QPainter::Antialiasing );
-            painter.setCompositionMode(QPainter::CompositionMode_Source);
-            painter.setPen( Qt::NoPen );
+            radialGradient.setColorAt(1, gradientStopColor( shadowColor, 0 ) );
 
-            const QRectF rect( QPoint( 0, 0 ), pixmap.size()/_helper.devicePixelRatio( pixmap ) );
-            painter.fillRect( rect, radialGradient );
+            // fill
+            QPainter p(&pixmap);
+            p.setCompositionMode(QPainter::CompositionMode_Source);
+            p.fillRect( pixmap.rect(), radialGradient);
 
-            painter.end();
+            // contrast pixel
+            p.setBrush( Qt::NoBrush );
+            p.setPen( gradientStopColor(shadowColor, shadowStrength) );
+            p.setRenderHints(QPainter::Antialiasing );
+            p.drawRoundedRect( QRect( shadowSize-shadowOffset, shadowSize-shadowOffset, shadowOffset, shadowOffset ), 3, 3 );
+
+            p.end();
 
             // create tiles from pixmap
-            _shadowTiles = TileSet( pixmap, Metrics::Shadow_Size, Metrics::Shadow_Size, 1, 1 );
+            _shadowTiles = TileSet( pixmap,
+                shadowSize-shadowOffset+Metrics::Shadow_Overlap,
+                shadowSize-shadowOffset+Metrics::Shadow_Overlap,
+                shadowOffset - 2*Metrics::Shadow_Overlap,
+                shadowOffset - 2*Metrics::Shadow_Overlap );
 
         }
 
@@ -186,57 +211,6 @@ namespace Breeze
     void ShadowHelper::objectDeleted( QObject* object )
     { _widgets.remove( static_cast<QWidget*>( object ) ); }
 
-    //_______________________________________________________
-    bool ShadowHelper::checkSupported( void ) const
-    {
-
-        // create atom
-        #if BREEZE_HAVE_X11
-
-        // make sure we are on X11
-        if( !Helper::isX11() ) return false;
-
-        // create atom
-        xcb_atom_t netSupportedAtom( _helper.createAtom( "_NET_SUPPORTED" ) );
-        if( !netSupportedAtom ) return false;
-
-        // store connection locally
-        xcb_connection_t* connection( Helper::connection() );
-
-        // get property
-        const quint32 maxLength = std::string().max_size();
-        xcb_get_property_cookie_t cookie( xcb_get_property( connection, 0, QX11Info::appRootWindow(), netSupportedAtom, XCB_ATOM_ATOM, 0, (maxLength+3) / 4 ) );
-        ScopedPointer<xcb_get_property_reply_t> reply( xcb_get_property_reply( connection, cookie, nullptr ) );
-        if( !reply ) return false;
-
-        // get reply length and data
-        const int count( xcb_get_property_value_length( reply.data() )/sizeof( xcb_atom_t ) );
-        xcb_atom_t *atoms = reinterpret_cast<xcb_atom_t*>( xcb_get_property_value( reply.data() ) );
-
-        bool found( false );
-        for( int i = 0; i < count && !found; ++i )
-        {
-            // get atom name and print
-            xcb_atom_t atom( atoms[i] );
-
-            xcb_get_atom_name_cookie_t cookie( xcb_get_atom_name( connection, atom ) );
-            ScopedPointer<xcb_get_atom_name_reply_t> reply( xcb_get_atom_name_reply( connection, cookie, 0 ) );
-            if( !reply ) continue;
-
-            // get name and compare
-            const QString name( QByteArray( xcb_get_atom_name_name( reply.data() ), xcb_get_atom_name_name_length( reply.data() ) ) );
-            if( strcmp( netWMShadowAtomName, xcb_get_atom_name_name( reply.data() ) ) == 0 ) found = true;
-
-        }
-
-        return found;
-
-        #else
-        return false;
-        #endif
-
-    }
-
     //_______________________________________________________
     bool ShadowHelper::isMenu( QWidget* widget ) const
     { return qobject_cast<QMenu*>( widget ); }
@@ -360,10 +334,6 @@ namespace Breeze
     //_______________________________________________________
     bool ShadowHelper::installX11Shadows( QWidget* widget )
     {
-
-        // do nothing if not supported
-        if( !_supported ) return false;
-
         // check widget and shadow
         if( !widget ) return false;
         if( !Helper::isX11() ) return false;
@@ -388,36 +358,20 @@ namespace Breeze
         foreach( const quint32& value, pixmaps )
         { data.append( value ); }
 
-
         // get devicePixelRatio
         // for testing purposes only
         const qreal devicePixelRatio( _helper.devicePixelRatio( _shadowTiles.pixmap( 0 ) ) );
 
+        // metrics
+        const int shadowSize = StyleConfigData::shadowSize()*12/16;
+        const int shadowOffset = qMax( shadowSize/2, Metrics::Shadow_Overlap*2 );
+
         // define shadows padding
-        int size( Metrics::Shadow_Size - Metrics::Shadow_Overlap );
-        int topSize = ( size - Metrics::Shadow_Offset ) * devicePixelRatio;
+        int size( shadowSize - Metrics::Shadow_Overlap );
+        int topSize = ( size - shadowOffset ) * devicePixelRatio;
         int bottomSize = size * devicePixelRatio;
-        int rightSize(0);
-        int leftSize(0);
-
-        switch( StyleConfigData::lightSource() )
-        {
-            case StyleConfigData::LS_TOPLEFT:
-            leftSize = (size - Metrics::Shadow_Offset) * devicePixelRatio;
-            rightSize = size * devicePixelRatio;
-            break;
-
-            case StyleConfigData::LS_TOPRIGHT:
-            rightSize = (size - Metrics::Shadow_Offset) * devicePixelRatio;
-            leftSize = size * devicePixelRatio;
-            break;
-
-            case StyleConfigData::LS_TOP:
-            leftSize = (size - Metrics::Shadow_Offset/2) * devicePixelRatio;
-            rightSize = (size - Metrics::Shadow_Offset/2) * devicePixelRatio;
-            break;
-
-        }
+        const int leftSize( (size - shadowOffset) * devicePixelRatio );
+        const int rightSize( size * devicePixelRatio );
 
         if( widget->inherits( "QBalloonTip" ) )
         {
@@ -455,7 +409,6 @@ namespace Breeze
     {
 
         #if BREEZE_HAVE_X11
-        if( !_supported ) return;
         if( !Helper::isX11() ) return;
         if( !( widget && widget->testAttribute(Qt::WA_WState_Created) ) ) return;
         xcb_delete_property( Helper::connection(), widget->winId(), _atom);
